#include "types.h"
#include "console.h"
#include "idt.h"
#include "inline_asm.h"
#include "keyboard.h"
#include "timer.h"
#include "gdt.h"

#include "sched.h"
#include "process.h"
#include "segment.h"

#include "user_tasks.h"
#include "init.h"


void start_kernel(void)
{
	char str[] = "C START!";
		
	/* screen - message display */
	init_screen();

	caos_printf("%s\n", str);

	/* exception & interrupt */
	init_idt();
	set_idtr();

	/* task management */
	init_task_table();

	init_cpu_tss();

	/* init user processes */
	create_task(init, INIT_ADDR, INIT_PCB, "init");
	create_task(user1, USER1_ADDR, USER1_PCB, "user1");
	create_task(user2, USER2_ADDR, USER2_PCB, "user2");


	/* scheduling */
	init_scheduler();


	/* device & IRQ */
	init_keyboard();
	init_timer();


	caos_printf("CaOS KERNEL END!!\n");

	/* execute user mode task */
	start_init();

	while (1);
}
		
		
		
