
#include "process.h"
#include "console.h"
#include "segment.h"
#include "types.h"


tss_struct *cpu_tss;
task_struct *task_table[MAX_TASK];

unsigned int task_count;


//
// clear every entry
//
void init_task_table(void)
{
	int l;
	for (l=0; l<MAX_TASK;l++) {
		task_table[l] = NULL;
	}

	task_count = 0;

}

//
// find empty entry and store new
// , return pid (process ID)
//
int add_task_table(task_struct *new)
{
	int l = 0;

	for (l=0; l<MAX_TASK;l++) {
		if (task_table[l] == NULL) {
			task_table[l] = new;
			return l;
		}
	}

	return -1;

}

//
// remove id(th) entry
//
int del_task_table(int id)
{
	if (task_table[id] == NULL)
		return -1;
	task_table[id] = NULL;
	return 1;
}



//
// func : task function pointer
// task_addr : user address space task function is copied 
// pcb_addr : address of task_struct
// name : task name
//
int create_task(void (*func)(void), unsigned int task_addr, unsigned int pcb_addr, char *name)
{
	int l;
	char *src, *dst;
	task_struct *task;

	caos_printf("Create task [%s]...", name);

	task = (task_struct *)pcb_addr;
	task->pid = add_task_table(task);

	if (task->pid < 0) {
		caos_printf("create_task fail\n");
		return -1;
	}

	task->status = PROCESS_RUNNING;
	task->task_addr = task_addr;

	src = (char *)func;
	dst = (char *)task_addr;
	for (l=0 ; l < PAGE_SIZE; l++) // copy one page
		*dst++ = *src++;


	task->thread.eip = task_addr;
	task->thread.esp = task_addr+PAGE_SIZE;
	task->thread.esp0 = pcb_addr+PAGE_SIZE;

	task->thread.cs = __USER_CS;
	task->thread.eflags = 0x200;
	task->thread.ss = __USER_DS;
	task->thread.ds = __USER_DS;
	task->thread.es = __USER_DS;
	task->thread.fs = __USER_DS;
	task->thread.gs = __USER_DS;

	for (l=0; name[l] != '\0'; l++)
		task->name[l] = name[l];

	caos_printf("%s (PID=%d) OK\n", task->name, task->pid);

	task_count++;

	caos_printf("Task count=%d\n", task_count);
	return task->pid;
}



//
// TSS segment must be initialized to run user-mode tasks
//
void init_cpu_tss(void)
{
	//
	// read pointer of TSS from 0x9000, somewhere over 0x9XXX
	//
	unsigned int tmp = *(unsigned int *)TSS_WHERE;
	
	// TSS for CPU
	cpu_tss = (tss_struct *)tmp;


	caos_printf("CPU TSS is at->[%x]\n", (unsigned int)cpu_tss);

	// Kernel Stack segment = 0x10
	cpu_tss->ss0 = __KERNEL_DS;
	
	asm volatile (
		"cli							\n\t"
		"mov $0x20, %%ax	\n\t"
		"ltr %%ax			\n\t"
		"sti				\n\t"
		::
	);
}

//
// execute user-mode task, init
//
void start_init(void)
{	
	task_struct *init_pcb = task_table[0];
	unsigned int reg;
   
	reg = (unsigned int)&(init_pcb->thread);
	

	asm volatile (
		"cli							\n\t"
		/* 
		 * Before task start, TSS must be initialized.
		 * Therefore kernel stack pointer is store at cpu_tss->esp0.
		*/ 
		"mov %1, %%eax		\n\t"
		"mov %%eax, %0		\n\t"
		"mov %2, %%esp		\n\t"
		"popa				\n\t"
		"pop %%ds				\n\t"
		"pop %%es				\n\t"
		"pop %%fs				\n\t"
		"pop %%gs				\n\t"
		"sti						\n\t"
		"iret						\n\t"
		: "=m"(cpu_tss->esp0)
		: "m"(init_pcb->thread.esp0),"m"(reg)
	);
}
		

